structure Test = 
struct


(** Required code to tie the lexer and the parser together **)
structure CalcLrVals = CalcLrValsFun (structure Token = LrParser.Token)

structure CalcLex = CalcLexFun (structure Tokens = CalcLrVals.Tokens)

structure CalcParser = 
  Join (structure LrParser = LrParser
        structure ParserData = CalcLrVals.ParserData
        structure Lex = CalcLex)

fun invoke lexstream =
    let fun print_error (s,i:int,_) =
            TextIO.output(TextIO.stdOut,
                          "Error, line " ^ (Int.toString i) ^ ", " ^ s ^ "\n")
     in CalcParser.parse(0,lexstream,print_error,())
    end


fun parse () = 
    let val lexer = CalcParser.makeLexer
                      (fn _ => valOf (TextIO.inputLine TextIO.stdIn))
        val dummyEOF = CalcLrVals.Tokens.EOF(0,0)
        val dummySTOP = CalcLrVals.Tokens.STOP(0,0)
        fun loop lexer =
            let val _ = print ">>>"
		val (result,lexer) = invoke lexer
                val (nextToken,lexer) = CalcParser.Stream.get lexer
             in case result
                  of SOME r => 
		     let
                       val _ = print ("exp = " ^ (Ast.toPrettyString r) ^ "\n")
		       val _ = print ("big step eval  = " ^ Ast.toPrettyString (Ast.evalBigStep r) ^ "\n") 
			       handle Ast.RunTimeTypeError  => print "Big Step Eval: Expressions is not well-formed (type mismatch).\n"

		       val _ = print ("small step eval = " ^ Ast.toPrettyString (Ast.evalSmallStep r) ^ "\n")
			       handle Ast.RunTimeTypeError  => print "Small Step Eval: Expressions is not well-formed (type mismatch).\n"
		     in
			 ()
		     end

                   | NONE => ();
                if CalcParser.sameToken(nextToken,dummyEOF) 
		   orelse
		   CalcParser.sameToken(nextToken,dummySTOP) 
		then ()
                else loop lexer
            end
     in loop lexer
    end

end

(*
open CalcLex
open UserDeclarations

fun test () = 
  let
     val fileName = "stuff"
     val file = TextIO.openIn fileName
    fun get _ = TextIO.input file
    val lexer: unit -> Internal.result = CalcLex.makeLexer get
  in
      case lexer () of 
	  EOF => print "DONE\n"
	| DIV => print "\\"
	| EOS => print ";"
	| ID s => print s 
	|  NUM i => print (Int.toString i)
	| LPAREN => print "("
	| PLUS => print "+"
	| SUB => print "-"
	| TIMES => print "*"
	| PRINT => print "print"
  end

end
*)
